# Generated by Django 2.2.6 on 2019-10-18 13:16

from django.db import migrations, models
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0064_auto_20191018_1159'),
    ]

    operations = [
        migrations.CreateModel(
            name='Dogovors_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('isFolder', models.BooleanField()),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('size', models.BigIntegerField(default=0, verbose_name='Размер фала')),
                ('real_name', models.TextField(blank=True, default=None, null=True, verbose_name='Первоначальное имя файла')),
                ('mime_type', isc_common.fields.name_field.NameField(blank=True, default=None, null=True, verbose_name='MIME тип файла файла')),
                ('format', isc_common.fields.name_field.NameField(blank=True, null=True, verbose_name='Формат файла')),
                ('date', models.DateTimeField(db_index=True, verbose_name='Дата')),
                ('date_sign', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата подписания')),
            ],
            options={
                'verbose_name': 'Документы',
                'db_table': 'sales_dogovors_view',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Precent_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('isFolder', models.BooleanField()),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('date', models.DateTimeField(db_index=True, verbose_name='Дата')),
                ('date_sign', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата подписания')),
                ('size', models.BigIntegerField(default=0, verbose_name='Размер фала')),
                ('real_name', models.TextField(blank=True, default=None, null=True, verbose_name='Первоначальное имя файла')),
                ('mime_type', isc_common.fields.name_field.NameField(blank=True, default=None, null=True, verbose_name='MIME тип файла файла')),
                ('format', isc_common.fields.name_field.NameField(blank=True, null=True, verbose_name='Формат файла')),
            ],
            options={
                'verbose_name': 'Документы',
                'db_table': 'sales_precent_view',
                'managed': False,
            },
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-16 16:28

from django.db import migrations, models
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0042_rec_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='status',
            name='id_owner',
            field=models.PositiveIntegerField(default=None),
        ),
        migrations.AddField(
            model_name='status',
            name='code_owner',
            field=isc_common.fields.code_field.CodeField(),
        ),
        migrations.AddField(
            model_name='status',
            name='name_owner',
            field=isc_common.fields.code_field.CodeField(),
        ),
        migrations.AlterField(
            model_name='status',
            name='code',
            field=isc_common.fields.code_field.CodeField(),
        ),
        migrations.AlterUniqueTogether(
            name='status',
            unique_together={('code', 'code_owner')},
        ),
    ]



import logging

from django.db.models import BigIntegerField, BooleanField, PositiveIntegerField

from isc_common.fields.code_field import CodeField
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager
from isc_common.models.base_ref import Hierarcy
from isc_common.models.tree_audit import TreeAuditModelManager, TreeAuditModelQuerySet
from isc_common.number import DelProps
from kaf_pas.ckk.models.item import Item_add
from kaf_pas.ckk.models.item_line import Item_lineManager
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import Documents
from kaf_pas.production.models.launches import Launches

logger = logging.getLogger(__name__)


class Launch_item_viewQuerySet(TreeAuditModelQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Launch_item_viewManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'STMP_1_id': record.STMP_1.id if record.STMP_1 else None,
            'STMP_1__value_str': record.STMP_1.value_str if record.STMP_1 else None,
            'STMP_2_id': record.STMP_2.id if record.STMP_2 else None,
            'STMP_2__value_str': record.STMP_2.value_str if record.STMP_2 else None,
            'lastmodified': record.lastmodified,
            'document_id': record.document.id if record.document else None,
            'document__file_document': record.document.file_document if record.document else None,
            'parent_id': record.parent_id,
            'launch_id': record.launch.id,
            'item_refs_id': record.item_refs_id,
            'editing': record.editing,
            'deliting': record.deliting,
            'isFolder': record.isFolder,
            'relevant': record.relevant,
            'version': record.version,
            'where_from': record.where_from,
            'props': record.props,
            'used': record.used,
            'icon': Item_lineManager.getIcon(record),
        }
        return DelProps(res)

    def updateFromRequest(self, request, removed=None, function=None):
        from kaf_pas.production.models.launch_item_refs import Launch_item_refs
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        used = data.get('used', False)
        if used == True:
            Launch_item_refs.objects.update_or_create(id=data.get('item_refs_id'), defaults=dict(props=Launch_item_refs.props.enabled))
        else:
            Launch_item_refs.objects.update_or_create(id=data.get('item_refs_id'), defaults=dict(props=~Launch_item_refs.props.enabled))

        return data

    def get_queryset(self):
        return Launch_item_viewQuerySet(self.model, using=self._db)


class Launch_item_view(Hierarcy):
    STMP_1 = ForeignKeyProtect(Document_attributes, verbose_name='Наименование изделия', related_name='STMP_1_view_launch', null=True, blank=True)
    STMP_2 = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение изделия', related_name='STMP_2_view_launch', null=True, blank=True)
    document = ForeignKeyProtect(Documents, verbose_name='Документ', null=True, blank=True)
    relevant = NameField()
    where_from = NameField()
    props = Item_add.get_prop_field()
    version = PositiveIntegerField(null=True, blank=True)
    section = CodeField(null=True, blank=True)
    item_refs_id = BigIntegerField()
    launch = ForeignKeyProtect(Launches, related_name='production_launch')

    isFolder = BooleanField()
    used = BooleanField()

    objects = Launch_item_viewManager()
    tree_objects = TreeAuditModelManager()

    def __str__(self):
        return f"ID: {self.id} STMP_1: [{self.STMP_1}], STMP_2: [{self.STMP_2}], props: {self.props}"

    class Meta:
        managed = False
        db_table = 'production_launch_item_view'
        verbose_name = 'Товарная позиция в производственной спецификации'



# Generated by Django 2.2.1 on 2019-05-30 12:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0043_auto_20190530_1205'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations',
            name='end',
            field=models.DateTimeField(blank=True, db_index=True, null=True),
        ),
    ]



import json
import logging
from tempfile import TemporaryFile

import dateutil
from django.core.files import File
from websocket import create_connection

from crypto.models.upload_file import DSResponse_UploadFile
from isc_common.datetime import StrToDate
from isc_common.dropzone import Dz
from isc_common.http.DSResponse import JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from kaf_pas.common.UploadItem import UploadItem
from kaf_pas.sales.models.customer import Customer
from kaf_pas.sales.models.dogovor_types import Dogovor_types
from kaf_pas.sales.models.dogovors import Dogovors
from kaf_pas.sales.models.status_dogovor import StatusDogovor

logger = logging.getLogger(__name__)


@JsonResponseWithException()
class DSResponse__Dogovor_UploadFile(DSResponse_UploadFile):

    def __init__(self, request):
        # raise Exception ('Its me !!!!!')
        self.id = request.GET.get('id')
        if request.GET.get('from_grid') == 'true':
            self.from_grid = True
        else:
            self.from_grid = False

        self.dictionary = dict(
            id=request.GET.get('id'),
            name=request.GET.get('name'),
            description=request.GET.get('description'),
            date=request.GET.get('date'),
            date_sign=request.GET.get('date_sign'),
            status_id=request.GET.get('status_id'),
            customer_id=request.GET.get('customer_id'),
            type_id=request.GET.get('type_id'),
        )

        self.dz_dictionary = Dz(request.POST)

        self.host = request.GET.get('host')
        self.port = request.GET.get('port')
        if not self.port:
            self.port = 80
        self.channel = request.GET.get('ws_channel')
        self.file = request.FILES.get('upload_attatch')

        self.handle_uploaded_file()

    @property
    def first_chunk(self):
        res = int(self.dz_dictionary.dzchunkindex) == 0
        return res

    @property
    def last_chunk(self):
        res = int(self.dz_dictionary.dztotalchunkcount) == int(self.dz_dictionary.dzchunkindex) + 1
        return res

    def handle_uploaded_file(self):
        try:
            self.dictionary.update(dict(
                real_file_name=self.file.name,
                stored_file_name=self.dz_dictionary.dzuuid,
                file_size=int(self.dz_dictionary.dztotalfilesize),
                file_mime_type=self.file.content_type)
            )

            item = UploadItem(dictionary=self.dictionary)
            old_path = None

            def load_str(pers):
                return f'Загружено: {pers} %'

            with TemporaryFile() as src:
                src.seek(int(self.dz_dictionary.dzchunkbyteoffset))
                src.write(self.file.read())

                pers = round((int(self.dz_dictionary.dzchunkindex) * 100) / int(self.dz_dictionary.dztotalchunkcount), 2)
                if self.last_chunk:
                    pers = 100

                logger.debug(f'{load_str(pers)}, шаг : {int(self.dz_dictionary.dzchunkindex) + 1} из {self.dz_dictionary.dztotalchunkcount}')

                if self.last_chunk:
                    logger.debug(f'Загружен файл: {item.real_file_name}.')
                    logger.debug(f'Запись временного файла.')
                    with open(item.full_path, 'w+b') as destination:
                        src.seek(0)
                        destination.write(src.read())
                    logger.debug(f'Запись временного файла выполнена.')

                    logger.debug(f'Запись: {item.full_path}.')
                    with open(item.full_path, 'rb') as src:
                        fileObj = File(src)

                        try:
                            old_path = Dogovors.objects.get(id=item.id).attfile.name
                        except Dogovors.DoesNotExist:
                            pass

                        res, created = Dogovors.objects.update_or_create(
                            id=item.id,
                            defaults=dict(
                                name=item.name,
                                description=item.description,
                                date=StrToDate(item.date, '%d.%m.%Y %H:%M:%S') if item.date else None,
                                date_sign=StrToDate(item.date_sign, '%d.%m.%Y %H:%M:%S') if item.date_sign else None,
                                status=StatusDogovor.objects.get(id=item.status_id),
                                customer=Customer.objects.get(id=item.customer_id),
                                type=Dogovor_types.objects.get(id=item.type_id),
                                attfile=fileObj,
                                file_store=self.get_path(fileObj.name),
                                format=item.file_format,
                                mime_type=item.file_mime_type,
                                size=item.file_size,
                                real_name=item.real_file_name,
                            )
                        )

                        ws = create_connection(f"ws://{self.host}:{self.port}/ws/{self.channel}/")
                        ws.send(json.dumps(dict(id=res.id, type="uploaded")))
                        ws.close()

                        logger.debug(f'Запись: {item.real_file_name} ({fileObj.name}) завершена.')

                    if old_path:
                        self.remove(old_path)
                        logger.debug(f'Удаление: {old_path} завершено.')

                    self.remove(item.full_path)
                    logger.debug(f'Удаление: {item.full_path} завершено.')

            self.response = dict(status=RPCResponseConstant.statusSuccess)

        except Exception as e:
            raise e



from django.urls import path

from kaf_pas.ckk.views import material_params

urlpatterns = [

    path('Material_params/Fetch/', material_params.Material_params_Fetch),
    path('Material_params/Add', material_params.Material_params_Add),
    path('Material_params/Update', material_params.Material_params_Update),
    path('Material_params/Remove', material_params.Material_params_Remove),
    path('Material_params/Lookup/', material_params.Material_params_Lookup),
    path('Material_params/Info/', material_params.Material_params_Info),
    path('Material_params/Copy', material_params.Material_params_Copy),

]



# Generated by Django 3.0.3 on 2020-02-16 20:13

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0185_auto_20200216_2008'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ready_2_launch_detail',
            name='ready',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='production.Ready_2_launch'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-06 08:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('system', '0001_initial'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Operation_types',
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-06 13:15

import bitfield.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('planing', '0025_auto_20191206_1151'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operations_users',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('props', bitfield.models.BitField((('compulsory_reading', 'Обязательное прочтение'),), db_index=True, default=1)),
                ('operation', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='planing.Operations')),
                ('user', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Ответствкнные по операциям планирования',
            },
        ),
    ]




from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.ckk.models.material_added_values import Material_added_values, Material_added_valuesManager


@JsonResponseWithException()
def Material_added_values_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Material_added_values.objects.
                get_range_rows1(
                request=request,
                # function=Material_added_valuesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_added_values_Add(request):
    return JsonResponse(DSResponseAdd(data=Material_added_values.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_added_values_Update(request):
    return JsonResponse(DSResponseUpdate(data=Material_added_values.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_added_values_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Material_added_values.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_added_values_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Material_added_values.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_added_values_Info(request):
    return JsonResponse(DSResponse(request=request, data=Material_added_values.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Material_added_values_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Material_added_values.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)            



# Generated by Django 2.2.7 on 2019-11-17 04:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0111_launch_item_line'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='launch_item_refs',
            name='child',
        ),
        migrations.RemoveField(
            model_name='launch_item_refs',
            name='item_refs',
        ),
        migrations.RemoveField(
            model_name='launch_item_refs',
            name='parent',
        ),
        migrations.RemoveField(
            model_name='launch_items',
            name='item',
        ),
        migrations.RemoveField(
            model_name='launch_items',
            name='launch',
        ),
        migrations.RemoveField(
            model_name='launches',
            name='demand',
        ),
        migrations.RemoveField(
            model_name='launches',
            name='parent',
        ),
        migrations.RemoveField(
            model_name='launches',
            name='status',
        ),
        migrations.DeleteModel(
            name='Launch_item_line',
        ),
        migrations.DeleteModel(
            name='Launch_item_refs',
        ),
        migrations.DeleteModel(
            name='Launch_items',
        ),
        migrations.DeleteModel(
            name='Launches',
        ),
    ]



import logging

from bitfield import BitField
from django.db import connection, transaction
from django.db.models import Q, UniqueConstraint, CheckConstraint, F

from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditQuerySet, AuditManager
from isc_common.models.base_ref import Hierarcy
from isc_common.models.tree_audit import TreeAuditModelManager
from isc_common.number import DelProps
from kaf_pas.ckk.models.item import Item

logger = logging.getLogger(__name__)


class Item_refsQuerySet(AuditQuerySet):
    def has_child_entered(self, id):
        with connection.cursor() as cursor:
            cursor.execute('''select count(*)
                                   from ckk_item_refs
                                   where child_id = %s''', [id])
            cnt, = cursor.fetchone()
            return cnt > 0

    def delete(self):
        from kaf_pas.ckk.models.item_qty import Item_qty
        with transaction.atomic():
            for item_refs in self:
                Item_qty.objects.filter(child=item_refs.child, parent=item_refs.parent).delete()
            return super().delete()

    def delete_with_checked(self):
        from kaf_pas.ckk.views.item_view import audo_top_level
        from kaf_pas.ckk.models.item_qty import Item_qty

        with transaction.atomic():
            for item_refs in self:
                Item_qty.objects.filter(child=item_refs.child, parent=item_refs.parent).delete()
                if not self.has_child_entered(id=item_refs.child.id):
                    Item_refs.objects1.create(child=item_refs.child, parent_id=int(audo_top_level))
            return super().delete()

    def get_or_create(self, defaults=None, **kwargs):
        return super().get_or_create(defaults=defaults, **kwargs)


class Item_refsManager(AuditManager):
    @staticmethod
    def props():
        return BitField(flags=(
            ('relevant', 'Актуальность'),  # 1
            ('used', 'Используемость в технологической спецификации'),  # 2
        ), default=3, db_index=True)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'child_id': record.child.id,
            'parent_id': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
            'used': record.props.used,
        }
        return DelProps(res)

    def get_queryset(self):
        return Item_refsQuerySet(self.model, using=self._db)


class Item_refs(Hierarcy):
    child = ForeignKeyProtect(Item, related_name='child')
    parent = ForeignKeyProtect(Item, related_name='parent', blank=True, null=True)

    # def get_parent(self, parent, path_array):
    #     path_array.append(parent.item_name)
    #     self.get_parent(parent.parent)
    #
    # def full_path(self):
    #     path_array = [self.child.item_name]
    #     parent

    props = Item_refsManager.props()

    objects = TreeAuditModelManager()
    objects1 = Item_refsManager()

    def __str__(self):
        return f'\nID={self.id}, child=[{self.child}], parent=[{self.parent}]'

    class Meta:
        verbose_name = 'Item_refs'
        constraints = [
            CheckConstraint(check=~Q(child=F('parent')), name='Item_refs_not_circulate_refs'),
            UniqueConstraint(fields=['child', 'parent'], name='Item_refs_unique_1'),
            UniqueConstraint(fields=['child'], condition=Q(parent=None), name='Item_refs_unique_2'),
        ]



# Generated by Django 2.2.3 on 2019-07-09 21:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0055_auto_20190709_2116'),
    ]

    operations = [
        migrations.AddField(
            model_name='uploades',
            name='log',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from clndr.models.calendars import get_graph
from isc_common.http.DSResponse import DSResponse, JsonWSPostResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse


@JsonWSPostResponseWithException()
def CalendarGraph_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=get_graph(request=request),
            status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.0.2 on 2020-01-20 15:15

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0168_auto_20200120_1028'),
    ]

    operations = [
        migrations.AddField(
            model_name='operation_def_resources',
            name='props',
            field=bitfield.models.BitField((('rec_default', 'Записать ресурс по умолчанию на все операции на которых нет ресурсов'),), db_index=True, default=0),
        ),
    ]



# Generated by Django 3.0.5 on 2020-04-21 10:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0225_item_document'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='item_document',
            constraint=models.UniqueConstraint(condition=models.Q(('document', None), ('lotsman_document', None)), fields=('item',), name='Item_document_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='item_document',
            constraint=models.UniqueConstraint(condition=models.Q(lotsman_document=None), fields=('document', 'item'), name='Item_document_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='item_document',
            constraint=models.UniqueConstraint(condition=models.Q(document=None), fields=('item', 'lotsman_document'), name='Item_document_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='item_document',
            constraint=models.UniqueConstraint(fields=('document', 'item', 'lotsman_document'), name='Item_document_unique_constraint_3'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-22 17:51

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0023_auto_20190522_1750'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_plan_template_items',
            name='item',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item'),
        ),
        migrations.AlterField(
            model_name='operation_plan_template_items',
            name='operation_plan_templates',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Operation_plan_templates'),
        ),
        migrations.AlterField(
            model_name='operation_plan_template_items',
            name='qty',
            field=models.PositiveIntegerField(),
        ),
        migrations.AlterField(
            model_name='operation_plan_templates_item_lines',
            name='operation_plan_template_item',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Operation_plan_template_items'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-08 14:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0203_auto_20200208_1408'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='material_askon',
            unique_together=set(),
        ),
    ]



# Generated by Django 2.2 on 2019-04-25 11:46

from django.db import migrations
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0038_auto_20190425_0829'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='STMP_2',
            field=isc_common.fields.name_field.NameField(verbose_name='Обозначение изделия'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-17 10:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0187_remove_ready_2_launch_detail_mat_view_name'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='resource',
            name='parent',
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import documents_view

urlpatterns = [

    path('Documents_view/Fetch/', documents_view.Documents_view_Fetch),
    path('Documents_bad_view/Fetch/', documents_view.Documents_bad_view_Fetch),
    path('Documents_view/Add', documents_view.Documents_view_Add),
    path('Documents_view/Update', documents_view.Documents_view_Update),
    path('Documents_view/Remove', documents_view.Documents_view_Remove),
    path('Documents_view/Lookup/', documents_view.Documents_view_Lookup),
    path('Documents_view/Info/', documents_view.Documents_view_Info),
    path('Documents_bad_view/Info/', documents_view.Documents_view_bad_Info),

]



# Generated by Django 2.2.6 on 2019-10-28 14:05

import django.utils.timezone
from django.db import migrations, models

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0095_auto_20191025_1311'),
        ('ckk', '0158_delete_status'),
        ('production', '0078_auto_20191022_0853'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations_item',
            name='item',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Item'),
        ),
        migrations.AlterUniqueTogether(
            name='operations_item',
            unique_together={('item', 'num')},
        ),
        migrations.CreateModel(
            name='Ready_2_launch',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('notes', models.TextField(blank=True, null=True)),
                ('demand', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='sales.Demand')),
            ],
            options={
                'verbose_name': 'Готовность к запуску',
            },
        ),
        migrations.RemoveField(
            model_name='operations_item',
            name='prod_item',
        ),
    ]



# Generated by Django 2.2 on 2019-04-14 08:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0013_auto_20190414_0758'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='document_attrs',
            name='deleted_at',
        ),
        migrations.RemoveField(
            model_name='documents',
            name='deleted_at',
        ),
        migrations.RemoveField(
            model_name='documents_thumb',
            name='deleted_at',
        ),
        migrations.RemoveField(
            model_name='documents_thumb10',
            name='deleted_at',
        ),
        migrations.RemoveField(
            model_name='pathes',
            name='deleted_at',
        ),
    ]



# Generated by Django 2.2 on 2019-04-26 09:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0010_operation_plan_template_locations_item_line'),
    ]

    operations = [
        migrations.CreateModel(
            name='TempProductionOperationsplan',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('qty', models.PositiveIntegerField()),
            ],
            options={
                'verbose_name': 'Временная таблица',
                'db_table': 'temp_production_operationsplan',
                'managed': False,
            },
        ),
    ]



from django.urls import path

from kaf_pas.production.views import ready_2_launch_detail

urlpatterns = [

    path('Ready_2_launch_detail/Fetch/', ready_2_launch_detail.Ready_2_launch_detail_Fetch),
    path('Ready_2_launch_detail/Add', ready_2_launch_detail.Ready_2_launch_detail_Add),
    path('Ready_2_launch_detail/Update', ready_2_launch_detail.Ready_2_launch_detail_Update),
    path('Ready_2_launch_detail/Remove', ready_2_launch_detail.Ready_2_launch_detail_Remove),
    path('Ready_2_launch_detail/Lookup/', ready_2_launch_detail.Ready_2_launch_detail_Lookup),
    path('Ready_2_launch_detail/Info/', ready_2_launch_detail.Ready_2_launch_detail_Info),
    path('Ready_2_launch_detail/Copy', ready_2_launch_detail.Ready_2_launch_detail_Copy),

]



import logging

from django.db.models import CharField, UniqueConstraint, Q
from isc_common.models.audit import AuditManager, AuditQuerySet
from isc_common.models.base_ref import Hierarcy

logger = logging.getLogger(__name__)


class Material_askonQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Material_askonManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'field0': record.field0,
            'field1': record.field1,
            'field2': record.field2,
            'field3': record.field3,
            'field4': record.field4,
            'field5': record.field5,
            'field6': record.field6,
            'field7': record.field7,
            'parent_id': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Material_askonQuerySet(self.model, using=self._db)


class Material_askon(Hierarcy):
    field0 = CharField(verbose_name='Информация в дерево материалов', db_index=True, max_length=255)
    field1 = CharField(verbose_name='Материал.Наименование и документ', null=True, blank=True, max_length=255)
    field2 = CharField(verbose_name='Материал.Код', null=True, blank=True, max_length=255)
    field3 = CharField(verbose_name='Сортамент.Наименование и документ', null=True, blank=True, max_length=255)
    field4 = CharField(verbose_name='Экземпляр сортамента.Типоразмер', null=True, blank=True, max_length=255)
    field5 = CharField(verbose_name='Экземпляр сортамента.Обозначение', null=True, blank=True, max_length=255)
    field6 = CharField(verbose_name='Экземпляр сортамента.Статус', null=True, blank=True, max_length=255)
    field7 = CharField(verbose_name='Экземпляр сортамента.Код', null=True, blank=True, max_length=255)
    objects = Material_askonManager()

    def __str__(self):
        return f"field0: {self.field0} field1: {self.field1} field2: {self.field2} field3: {self.field3} field4: {self.field4} field5: {self.field5} field6: {self.field6} field7: {self.field7}"

    class Meta:
        verbose_name = 'Материалы из Асона'
        constraints = [
            UniqueConstraint(fields=['id'], condition=Q(parent=None), name='Material_askon_unique_constraint_0'),
            UniqueConstraint(fields=['id', 'parent'], name='Material_askon_unique_constraint_1'),
        ]
        # unique_together = (('field0', 'field1', 'field2', 'field3', 'field4', 'field5', 'field6', 'field7'),)



class Dynamic:

    def get_datasources(self):
        from kaf_pas.ckk.models.material_params import Material_params
        from kaf_pas.ckk.models.material_type import Material_type
        from refs.models.type_param import Type_param

        def uncapitalize(str):
            return str[0:1].lower() + str[1:]

        data_sources = []
        for material_type in Material_type.objects.all():
            step = 0
            fields = []
            for param in Material_params.objects.filter(materials_type=material_type).order_by('order', ):
                if param.param_type.type == Type_param.integer():
                    fields.append(f'unstackedSpinnerField("{param.param_type.code}", "{param.param_type.name}", {uncapitalize(str(param.param_type.required))})')
                elif param.param_type.type == Type_param.float():
                    fields.append(f'floatField("{param.param_type.code}", "{param.param_type.name}", {uncapitalize(str(param.param_type.required))})')
                elif param.param_type.type == Type_param.date():
                    fields.append(f'dateField("{param.param_type.code}", "{param.param_type.name}", {uncapitalize(str(param.param_type.required))})')
                elif param.param_type.type == Type_param.date_time():
                    fields.append(f'dateTimeField("{param.param_type.code}", "{param.param_type.name}", {uncapitalize(str(param.param_type.required))})')
                elif param.param_type.type == Type_param.string():
                    fields.append(f'nameField("{param.param_type.code}", "{param.param_type.name}", {uncapitalize(str(param.param_type.required))}, false, {param.param_type.length if param.param_type.length else 255})')
                elif param.param_type.type == Type_param.select():
                    fields.append(f'selectField("{param.param_type.code}", "{param.param_type.name}", {param.param_type.value_map}, {uncapitalize(str(param.param_type.required))})')
                else:
                    raise Exception(f'{param.param_type.type}: unknown type.')

            step += 1
            identifier = material_type.code
            data_source = '''isc.RestDataSourceSS.create(
                {
                    identifier: "$identifier",
                    fields: $FIELDS,
                    addDataURL: "logic/Material_added_values/Add",
                    fetchDataURL: "logic/Material_added_values/Fetch/",
                    removeDataURL: "logic/Material_added_values/Remove",
                    updateDataURL: "logic/Material_added_values/Update",
                    lookupDataURL: "logic/Material_added_values/Lookup/",
                    infoDataURL: "logic/Material_added_values/Info/",
                    copyDataURL: "logic/Material_added_values/Copy",
                })
            '''
            data_source = data_source.replace('$FIELDS', f"[{','.join(fields)}]")
            data_source = data_source.replace('$identifier', identifier).replace(' ', '').replace('\n', '')
            data_sources.append(data_source)
        sql_str = ','.join(data_sources)
        sql_str = f'[{sql_str}]'
        return sql_str



# Generated by Django 2.2 on 2019-04-18 11:03

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0030_locations'),
    ]

    operations = [
        migrations.AddField(
            model_name='locations',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-06 09:23

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0021_auto_20191206_0855'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_types',
            name='props',
            field=bitfield.models.BitField((('isoper', 'Является операцией'), ('plus', 'Приходная операция'), ('minus', 'Приходная операция')), db_index=True, default=0),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-22 12:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0020_operation_material'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='operation_plan_templates',
            options={'verbose_name': 'Шаблоны заданий в производство'},
        ),
        migrations.RemoveField(
            model_name='operation_plan_templates',
            name='name',
        ),
        migrations.AddField(
            model_name='operation_plan_templates',
            name='qty',
            field=models.PositiveIntegerField(default=None),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-07 07:28

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        # ('one_c', '0015_delete_uuid_2_id'),
        ('ckk', '0126_remove_materials_material_1c'),
    ]

    operations = [
        migrations.AddField(
            model_name='materials',
            name='nomenklatura_1C',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='one_c.Document_1c'),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.document_attributes import Document_attributes, Document_attributesManager


@JsonResponseWithException()
def Document_attributes_SPC_CLM_FORMAT_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Document_attributes.objects.
                select_related('attr_type').
                filter(attr_type__code='SPC_CLM_FORMAT').
                get_range_rows1(
                request=request,
                function=Document_attributesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_SPC_CLM_FORMAT_Add(request):
    return JsonResponse(DSResponseAdd(data=Document_attributes.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_SPC_CLM_FORMAT_Update(request):
    return JsonResponse(DSResponseUpdate(data=Document_attributes.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_SPC_CLM_FORMAT_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Document_attributes.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_SPC_CLM_FORMAT_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Document_attributes.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_SPC_CLM_FORMAT_Info(request):
    return JsonResponse(DSResponse(request=request, data=Document_attributes.objects.
                                   select_related('attr_type').
                                   filter(attr_type__code='SPC_CLM_FORMAT').
                                   get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Document_attributes_SPC_CLM_FORMAT_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Document_attributes.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)            



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.sales.models.dogovor_types import Dogovor_types, Dogovor_typesManager


@JsonResponseWithException()
def Dogovor_types_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Dogovor_types.objects.
                filter(props=Dogovor_types.props.real).
                get_range_rows1(
                request=request,
                function=Dogovor_typesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovor_types_Add(request):
    return JsonResponse(DSResponseAdd(data=Dogovor_types.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovor_types_Update(request):
    return JsonResponse(DSResponseUpdate(data=Dogovor_types.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovor_types_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Dogovor_types.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovor_types_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Dogovor_types.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovor_types_Info(request):
    return JsonResponse(DSResponse(request=request, data=Dogovor_types.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Dogovor_types_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Dogovor_types.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.0.2 on 2020-01-15 09:59

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0123_pathes_attr_type'),
        ('ckk', '0181_auto_20191217_1331'),
    ]

    operations = [
        migrations.AddField(
            model_name='item_line',
            name='Документ_на_материал',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Документ_на_материал_Item_line', to='kd.Document_attributes'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='Документ_на_сортамент',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Документ_на_сортамент_Item_line', to='kd.Document_attributes'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='Наименование_материала',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Наименование_материала_Item_line', to='kd.Document_attributes'),
        ),
        migrations.AddField(
            model_name='item_line',
            name='Наименование_сортамента',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Наименование_сортамента_Item_line', to='kd.Document_attributes'),
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-22 07:52

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0034_standard_colors_color'),
        ('planing', '0082_operation_color_color'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_color',
            name='color',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='isc_common.Standard_colors'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-08 14:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0172_auto_20200208_1429'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='operations_item',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='operations_item',
            constraint=models.UniqueConstraint(condition=models.Q(num=None), fields=('item', 'operation'), name='Operations_item_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='operations_item',
            constraint=models.UniqueConstraint(fields=('item', 'num', 'operation'), name='Operations_item_unique_constraint_1'),
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException, JsonWSResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.production.models.ready_2_launch import Ready_2_launch, Ready_2_launchManager


@JsonResponseWithException()
def Ready_2_launch_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Ready_2_launch.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Ready_2_launchManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_Add(request):
    return JsonResponse(DSResponseAdd(data=Ready_2_launch.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_Update(request):
    return JsonResponse(DSResponseUpdate(data=Ready_2_launch.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Ready_2_launch.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Ready_2_launch.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_Info(request):
    return JsonResponse(DSResponse(request=request, data=Ready_2_launch.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Ready_2_launch.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.7 on 2019-11-20 14:41

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0125_auto_20191119_1843'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_line',
            name='added_parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_item_added_parent', to='production.Launch_item_line'),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-20 18:57

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0146_material_askon_mview'),
        ('production', '0055_remove_operation_material_material'),
    ]

    operations = [
        migrations.AddField(
            model_name='operation_material',
            name='material',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Material_askon'),
        ),
    ]



import logging

from django.db.models import BooleanField, PositiveIntegerField

from isc_common import delAttr
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditQuerySet, AuditManager
from isc_common.models.base_ref import Hierarcy
from kaf_pas.ckk.models.item import Item_add
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
from kaf_pas.planing.models.operations_view import Operations_view

logger = logging.getLogger(__name__)


class Item_flat_viewQuerySet(AuditQuerySet):

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)

    def get_infoPlan(self, request, *args):
        request = DSRequest(request=request)
        data = request.get_data()
        item_ids = [item.get('item_id') for item in Operations_view.objects.filter(
            launch_id=data.get('launch_id'),
            operation_level_id=data.get('level_id'),
            resource_id=data.get('resource_id'),
            resource__location_id=data.get('location_id'),
        ).values('item_id').distinct()]

        delAttr(request.json.get('data'), 'full_name')
        delAttr(request.json.get('data'), 'launch_id')
        delAttr(request.json.get('data'), 'level_id')
        delAttr(request.json.get('data'), 'resource_id')
        delAttr(request.json.get('data'), 'location_id')

        # criteria = self.get_criteria(json=request.json)
        cnt = super().filter(id__in=item_ids).count()
        cnt_all = super().filter().count()
        return dict(qty_rows=cnt, all_rows=cnt_all)


class Item_flat_viewManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'STMP_1_id': record.STMP_1.id if record.STMP_1 else None,
            'STMP_1__value_str': record.STMP_1.value_str if record.STMP_1 else None,
            'STMP_2_id': record.STMP_2.id if record.STMP_2 else None,
            'STMP_2__value_str': record.STMP_2.value_str if record.STMP_2 else None,
            'lastmodified': record.lastmodified,
            'document_id': record.document.id if record.document else None,
            'document__file_document': record.document.file_document if record.document else None,
            'lotsman_document_id': record.lotsman_document.id if record.lotsman_document else None,
            'editing': record.editing,
            'deliting': record.deliting,
            'isFolder': record.isFolder,
            'relevant': record.relevant,
            'confirmed': record.confirmed,
            'where_from': record.where_from,
            'version': record.version,
            'props': int(record.props),
        }
        # print(res)
        return res

    def get_queryset(self):
        return Item_flat_viewQuerySet(self.model, using=self._db)


class Item_flat_view(Hierarcy):
    STMP_1 = ForeignKeyProtect(Document_attributes, verbose_name='Наименование изделия', related_name='STMP_1_flat_view', null=True, blank=True)
    STMP_2 = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение изделия', related_name='STMP_2_flat_view', null=True, blank=True)
    document = ForeignKeyProtect(Documents, verbose_name='Документ', null=True, blank=True)
    lotsman_document = ForeignKeyProtect(Lotsman_documents_hierarcy, verbose_name='Документ', null=True, blank=True)
    relevant = NameField()
    confirmed = NameField()
    where_from = NameField()
    props = Item_add.get_prop_field()
    version = PositiveIntegerField(null=True, blank=True)

    isFolder = BooleanField()

    objects = Item_flat_viewManager()

    @property
    def item(self):
        from kaf_pas.ckk.models.item import Item
        return Item.objects.get(id=self.id)

    def __str__(self):
        return f"ID={self.id} STMP_1=[{self.STMP_1}], STMP_2=[{self.STMP_2}], props={self.props}"

    class Meta:
        managed = False
        db_table = 'ckk_item_flat_view'
        verbose_name = 'Товарная позиция'



import logging

from bitfield import BitHandler
from django.db.models import UniqueConstraint, Q
from django.forms import model_to_dict

from isc_common import setAttr, delAttr
from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.base_ref import BaseRefHierarcy, BaseRefManager, BaseRefQuerySet
from kaf_pas.ckk.models.locations import Locations
from kaf_pas.ckk.models.material_type import Material_type
from one_c.models.document_1c import Document_1c

logger = logging.getLogger(__name__)


class MaterialsQuerySet(BaseRefQuerySet, CommonManagetWithLookUpFieldsQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class MaterialsManager(BaseRefManager, CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        from kaf_pas.k_one_c.models.nomenklatura import Nomenklatura_model

        nomenklatura = None
        if record.nomenklatura_model:
            nomenklatura = Nomenklatura_model.objects.get(pk=record.nomenklatura_model.ref)

        res = {
            'id': record.id,
            'code': record.code,
            'gost': record.gost,
            'name': record.name,
            'full_name': record.full_name,
            'description': record.description,
            'location_id': record.location.id,
            'location__name': record.location.name,
            'location__full_name': record.location.full_name,
            'materials_type_id': record.materials_type.id,
            'materials_type__code': record.materials_type.code,
            'materials_type__name': record.materials_type.name,
            'materials_type__description': record.materials_type.description,
            'materials_type__full_name': record.materials_type.full_name,
            'parent_id': record.parent.id if record.parent else None,
            'nomenklatura_model_ref': nomenklatura.ref if nomenklatura else None,
            'nomenklatura_model__code': nomenklatura.code if nomenklatura else None,
            'nomenklatura_model__full_name': nomenklatura.full_name if nomenklatura else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return MaterialsQuerySet(self.model, using=self._db)

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        material_type = Material_type.objects.get(id=data.get('materials_type_id'))
        setAttr(_data, 'materials_type_id', material_type.id)

        if data.get('nomenklatura_model__full_name'):
            from kaf_pas.k_one_c.models.nomenklatura import Nomenklatura_model
            nomenklatura_model = Nomenklatura_model.objects.get(full_name=data.get('nomenklatura_model__full_name'))
            setAttr(_data, 'nomenklatura_model', Document_1c.objects.get(pk=nomenklatura_model.ref))

        delAttr(_data, 'ed_izm__code')
        delAttr(_data, 'location__full_name')
        delAttr(_data, 'materials_type__full_name')
        delAttr(_data, 'nomenklatura_model_id')
        delAttr(_data, 'nomenklatura_model_ref')
        delAttr(_data, 'nomenklatura_model__code')
        delAttr(_data, 'nomenklatura_model__full_name')

        res = super().create(**_data)
        res = model_to_dict(res)
        props = res.get( 'props')
        if props and isinstance(props, BitHandler):
            props = res.get( 'props')._value
            setAttr(res, 'props', props)
        data.update(res)
        return data

    def updateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        material_type = Material_type.objects.get(full_name=data.get('materials_type__full_name'))
        setAttr(_data, 'materials_type_id', material_type.id)

        if data.get('nomenklatura_model__full_name'):
            from kaf_pas.k_one_c.models.nomenklatura import Nomenklatura_model
            nomenklatura_model = Nomenklatura_model.objects.get(full_name=data.get('nomenklatura_model__full_name'))
            setAttr(_data, 'nomenklatura_model', Document_1c.objects.get(pk=nomenklatura_model.ref))

        delAttr(_data, 'ed_izm__code')
        delAttr(_data, 'location__full_name')
        delAttr(_data, 'materials_type__name')
        delAttr(_data, 'materials_type__code')
        delAttr(_data, 'materials_type__description')
        delAttr(_data, 'materials_type__full_name')
        delAttr(_data, 'nomenklatura_model_id')
        delAttr(_data, 'nomenklatura_model_ref')
        delAttr(_data, 'nomenklatura_model__code')
        delAttr(_data, 'nomenklatura_model__full_name')
        delAttr(_data, 'location')
        delAttr(_data, 'location__name')
        delAttr(_data, 'materials_type')
        delAttr(_data, 'full_name')
        delAttr(_data, 'isFolder')
        delAttr(_data, 'parent')
        delAttr(_data, 'id')

        res, created = super().filter(id=data.get('id')).update(**_data)
        res = model_to_dict(res)
        props = res.get( 'props')
        if props and isinstance(props, BitHandler):
            props = res.get( 'props')._value
            setAttr(res, 'props', props)
        data.update(res)
        return data


class Materials(BaseRefHierarcy):
    gost = CodeField()
    location = ForeignKeyProtect(Locations)
    materials_type = ForeignKeyProtect(Material_type)
    nomenklatura_model = ForeignKeyProtect(Document_1c, null=True, blank=True)
    objects = MaterialsManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Материалы'
        constraints = [
            UniqueConstraint(fields=['gost', 'location', 'materials_type'], condition=Q(name=None) & Q(nomenklatura_model=None), name='Materials_unique_constraint_0'),
            UniqueConstraint(fields=['gost', 'location', 'materials_type', 'name'], condition=Q(nomenklatura_model=None), name='Materials_unique_constraint_1'),
            UniqueConstraint(fields=['gost', 'location', 'materials_type', 'nomenklatura_model'], condition=Q(name=None), name='Materials_unique_constraint_2'),
            UniqueConstraint(fields=['gost', 'location', 'materials_type', 'name', 'nomenklatura_model'], name='Materials_unique_constraint_3'),
        ]



import logging

from django.db.models import BigIntegerField, BooleanField
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditQuerySet, AuditManager

from kaf_pas.ckk.models.item import Item_add
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import Documents

logger = logging.getLogger(__name__)


class Cdw_Pdf_Item_viewQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Cdw_Pdf_Item_viewManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "STMP_1_id": record.STMP_1.id if record.STMP_1 else None,
            "STMP_1__value_str": record.STMP_1.value_str if record.STMP_1 else None,
            "STMP_2_id": record.STMP_2.id if record.STMP_2 else None,
            "STMP_2__value_str": record.STMP_2.value_str if record.STMP_2 else None,
            "lastmodified": record.lastmodified,
            "document_id": record.document.id if record.document else None,
            "document__file_document": record.document.file_document if record.document else None,
            "parent_id": record.parent_id,
            "editing": record.editing,
            "deliting": record.deliting,
            "isFolder": record.isFolder,
            "relevant": record.relevant,
            "props": int(record.props),
        }
        # print(res)
        return res

    def get_queryset(self):
        return Cdw_Pdf_Item_viewQuerySet(self.model, using=self._db)


class Cdw_Pdf_Item_view(AuditModel):
    STMP_1 = ForeignKeyProtect(Document_attributes, verbose_name='Наименование изделия', related_name='STMP_1_cdw', null=True, blank=True)
    STMP_2 = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение изделия', related_name='STMP_2_cdw', null=True, blank=True)
    isFolder = BooleanField(verbose_name='Обозначение изделия')
    document = ForeignKeyProtect(Documents, verbose_name='Документ', null=True, blank=True)
    parent_id = BigIntegerField(null=True, blank=True)
    relevant = NameField()
    props = Item_add.get_prop_field()

    objects = Cdw_Pdf_Item_viewManager()


    def __str__(self):
        return f"{self.STMP_1}: {self.STMP_2} (props : {self.props} ,type document: {self.document.attr_type.code}, deleted_at: {self.deleted_at})"
        # return f"id: {self.id}, parent_id: {self.parent.id}, relevant: {self.relevant},  isFolder: {self.isFolder}"


    class Meta:
        managed = False
        # db_table = 'ckk_cdw_pdf_item_view'
        db_table = 'ckk_item_view'
        verbose_name = 'Чертеж'



# Generated by Django 3.0.3 on 2020-02-08 11:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0200_auto_20200208_0741'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='item_refs',
            name='Item_refs_not_circulate_refs',
        ),
        migrations.AlterUniqueTogether(
            name='item_refs',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('STMP_1', None), ('STMP_2', None), ('lotsman_type_id', None), ('version', None)), fields=('props',), name='Item_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('STMP_2', None), ('lotsman_type_id', None), ('version', None)), fields=('STMP_1', 'props'), name='Item_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('STMP_1', None), ('lotsman_type_id', None), ('version', None)), fields=('STMP_2', 'props'), name='Item_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('lotsman_type_id', None), ('version', None)), fields=('STMP_1', 'STMP_2', 'props'), name='Item_unique_constraint_3'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('STMP_1', None), ('STMP_2', None), ('lotsman_type_id', None)), fields=('props', 'version'), name='Item_unique_constraint_4'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('STMP_2', None), ('lotsman_type_id', None)), fields=('STMP_1', 'props', 'version'), name='Item_unique_constraint_5'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('STMP_1', None), ('lotsman_type_id', None)), fields=('STMP_2', 'props', 'version'), name='Item_unique_constraint_6'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(lotsman_type_id=None), fields=('STMP_1', 'STMP_2', 'props', 'version'), name='Item_unique_constraint_7'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('STMP_1', None), ('STMP_2', None), ('version', None)), fields=('lotsman_type_id', 'props'), name='Item_unique_constraint_8'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('STMP_2', None), ('version', None)), fields=('STMP_1', 'lotsman_type_id', 'props'), name='Item_unique_constraint_9'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('STMP_1', None), ('version', None)), fields=('STMP_2', 'lotsman_type_id', 'props'), name='Item_unique_constraint_10'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(version=None), fields=('STMP_1', 'STMP_2', 'lotsman_type_id', 'props'), name='Item_unique_constraint_11'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('STMP_1', None), ('STMP_2', None)), fields=('lotsman_type_id', 'props', 'version'), name='Item_unique_constraint_12'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(STMP_2=None), fields=('STMP_1', 'lotsman_type_id', 'props', 'version'), name='Item_unique_constraint_13'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(STMP_1=None), fields=('STMP_2', 'lotsman_type_id', 'props', 'version'), name='Item_unique_constraint_14'),
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(fields=('STMP_1', 'STMP_2', 'lotsman_type_id', 'props', 'version'), name='Item_unique_constraint_15'),
        ),
        migrations.AddConstraint(
            model_name='item_refs',
            constraint=models.UniqueConstraint(fields=('child', 'parent'), name='Item_refs_unique_1'),
        ),
        migrations.AddConstraint(
            model_name='item_refs',
            constraint=models.UniqueConstraint(condition=models.Q(parent=None), fields=('child',), name='Item_refs_unique_2'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-26 10:03

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0012_auto_20190426_1001'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operationsplan',
            name='status',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Status'),
        ),
        migrations.DeleteModel(
            name='Operation_plan_status',
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-19 03:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0099_auto_20200219_0316'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Operation_locations',
        ),
    ]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.lotsman_document_attr_cross import Lotsman_document_attr_cross


class Lotsman_document_attr_crossManager(object):
    pass


@JsonResponseWithException()
def Lotsman_document_attr_cross_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Lotsman_document_attr_cross.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Lotsman_document_attr_crossManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_document_attr_cross_Add(request):
    return JsonResponse(DSResponseAdd(data=Lotsman_document_attr_cross.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_document_attr_cross_Update(request):
    return JsonResponse(DSResponseUpdate(data=Lotsman_document_attr_cross.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_document_attr_cross_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Lotsman_document_attr_cross.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_document_attr_cross_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Lotsman_document_attr_cross.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_document_attr_cross_Info(request):
    return JsonResponse(DSResponse(request=request, data=Lotsman_document_attr_cross.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Lotsman_document_attr_cross_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Lotsman_document_attr_cross.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.1.7 on 2019-03-28 10:18

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0003_auto_20190327_1141'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document_attrs',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'),), blank=True, default=1, null=True),
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import operation_launches

urlpatterns = [

    path('Operation_launches/Fetch/', operation_launches.Operation_launches_Fetch),
    path('Operation_launches/Add', operation_launches.Operation_launches_Add),
    path('Operation_launches/Update', operation_launches.Operation_launches_Update),
    path('Operation_launches/MakeProdOrder', operation_launches.Operation_launches_MakeProdOrder),
    path('Operation_launches/DeleteProdOrder', operation_launches.Operation_launches_DeleteProdOrder),
    path('Operation_launches/ReCalcRoutes', operation_launches.Operation_launches_ReCalcRoutes),
    path('Operation_launches/CleanRoutes', operation_launches.Operation_launches_CleanRoutes),
    path('Operation_launches/Remove', operation_launches.Operation_launches_Remove),
    path('Operation_launches/Lookup/', operation_launches.Operation_launches_Lookup),
    path('Operation_launches/Info/', operation_launches.Operation_launches_Info),
    path('Operation_launches/Copy', operation_launches.Operation_launches_Copy),

]



# Generated by Django 3.0.2 on 2020-01-31 11:21

from django.db import migrations, models
import django.db.models.expressions


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0126_remove_lotsman_documents_hierarcy_path'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='lotsman_documents_hierarcy_refs',
            constraint=models.CheckConstraint(check=models.Q(_negated=True, child=django.db.models.expressions.F('parent')), name='Lotsman_documents_hierarcy_not_circulate_refs'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-22 08:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0158_delete_status'),
        ('production', '0077_delete_operations_refs'),
    ]

    operations = [
        migrations.RenameField(
            model_name='item_refs',
            old_name='item',
            new_name='item_refs',
        ),
        migrations.AlterUniqueTogether(
            name='item_refs',
            unique_together={('item_refs', 'child', 'parent')},
        ),
    ]



import logging

from kaf_pas.kd.models.documents import Documents, DocumentManager, DocumentQuerySet

logger = logging.getLogger(__name__)


class SpwQuerySet(DocumentQuerySet):
    pass


class SpwManager(DocumentManager):
    def get_queryset(self):
        return SpwQuerySet(self.model, using=self._db).filter(attr_type__code='SPW')


class Spws(Documents):
    objects = SpwManager()

    class Meta:
        verbose_name = 'Спецификации'
        proxy = True



# Generated by Django 2.2.3 on 2019-07-28 19:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0074_auto_20190728_1924'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='pathes',
            name='attr_type',
        ),
    ]



from django.urls import path

from kaf_pas.production.views import operations_template

urlpatterns = [

    path('Operations_template/Fetch/', operations_template.Operations_template_Fetch),
    path('Operations_template/Add', operations_template.Operations_template_Add),
    path('Operations_template/Update', operations_template.Operations_template_Update),
    path('Operations_template/Remove', operations_template.Operations_template_Remove),
    path('Operations_template/Lookup/', operations_template.Operations_template_Lookup),
    path('Operations_template/Info/', operations_template.Operations_template_Info),
    path('Operations_template/Copy', operations_template.Operations_template_Copy),

]



# Generated by Django 3.0.2 on 2020-01-21 22:31

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('planing', '0078_auto_20200121_2231'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations',
            name='creator',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='operations',
            name='date',
            field=models.DateTimeField(),
        ),
    ]



import logging

from django.core.management import BaseCommand
from django.db import transaction

from kaf_pas.production.models.ready_2_launch import Ready_2_launchManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Тестирование"

    def add_arguments(self, parser):
        parser.add_argument('--demand_id', type=int)
        parser.add_argument('--user_id', type=int)

    def handle(self, *args, **options):
        demand_id = options.get('demand_id')
        user_id = options.get('user_id')

        with transaction.atomic():
            Ready_2_launchManager.make(demand=demand_id, user=user_id)
        print('Done')



import logging

from isc_common.models.base_ref import BaseRef, BaseRefManager, BaseRefQuerySet

logger = logging.getLogger(__name__)


class Precent_item_typesQuerySet(BaseRefQuerySet):
    pass


class Precent_item_typesManager(BaseRefManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Precent_item_typesQuerySet(self.model, using=self._db)


class Precent_item_types(BaseRef):
    objects = Precent_item_typesManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}"

    class Meta:
        verbose_name = 'Типы приемки'



from django.urls import path

from kaf_pas.sales.views import demand

urlpatterns = [

    path('Demand/Fetch/', demand.Demand_Fetch),
    path('Demand/Add', demand.Demand_Add),
    path('Demand/Update', demand.Demand_Update),
    path('Demand/Remove', demand.Demand_Remove),
    path('Demand/Lookup/', demand.Demand_Lookup),
    path('Demand/Info/', demand.Demand_Info),
    path('Demand/Copy', demand.Demand_Copy),

]



# Generated by Django 3.0.3 on 2020-02-10 08:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0206_auto_20200209_1230'),
        ('production', '0176_auto_20200209_1207'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='operations_item',
            name='Operations_item_unique_constraint_0',
        ),
        migrations.RemoveConstraint(
            model_name='operations_item',
            name='Operations_item_unique_constraint_1',
        ),
        migrations.AlterUniqueTogether(
            name='operations_item',
            unique_together={('item', 'operation')},
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-09 12:07

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('production', '0175_auto_20200209_1152'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='resource_users',
            unique_together={('resource', 'user')},
        ),
        migrations.AddConstraint(
            model_name='resource',
            constraint=models.UniqueConstraint(condition=models.Q(calendar=None), fields=('location',), name='Resource_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='resource',
            constraint=models.UniqueConstraint(fields=('calendar', 'location'), name='Resource_unique_constraint_1'),
        ),
    ]



import logging

from bitfield import BitField, BitHandler
from django import db
from django.conf import settings
from django.db import transaction, connection
from django.db.models import PositiveIntegerField, BigIntegerField, UniqueConstraint, Q
from django.forms import model_to_dict

from isc_common import setAttr, delAttr
from isc_common.bit import TurnBitOn
from isc_common.common.functions import delete_dbl_spaces
from isc_common.common.mat_views import refresh_mat_view
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from isc_common.progress import managed_progress, ProgressDroped
from isc_common.string import get_NoneStr
from kaf_pas.ckk.models import get_operations_from_trunsaction
from kaf_pas.kd.models.document_attributes import Document_attributes, Document_attributesManager
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
from kaf_pas.production.models import progress_deleted

logger = logging.getLogger(__name__)


class ItemQuerySet(CommonManagetWithLookUpFieldsQuerySet):

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class ItemManager(CommonManagetWithLookUpFieldsManager):

    def createFromRequest(self, request):
        from kaf_pas.ckk.models.item_refs import Item_refs

        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        parent = _data.get('parent')
        if parent:
            delAttr(_data, 'parent')
            setAttr(_data, 'parent_id', parent)

        delAttr(_data, 'STMP_1__value_str')
        delAttr(_data, 'STMP_2__value_str')

        props = 0

        props |= Item.props.relevant
        props |= Item.props.man_input

        delAttr(_data, 'relevant')
        delAttr(_data, 'where_from')
        setAttr(_data, 'props', props)

        res = super().create(**_data)
        logger.debug(f'Added: {res}')

        item_refs, create = Item_refs.objects.get_or_create(parent_id=_data.get('parent_id'), child=res)
        if create:
            logger.debug(f'Added item_ref: {item_refs}')

        res = model_to_dict(res)

        props = res.get('props')
        if props and isinstance(props, BitHandler):
            props = res.get('props')._value
            setAttr(res, 'props', props)
        setAttr(res, 'isFolder', False)
        data.update(res)
        return data

    @staticmethod
    def check_4_production(item):
        if isinstance(item, int):
            id = item
        elif isinstance(item, Item):
            id = item.id
        else:
            raise Exception(f'item must be Item or int')

        with connection.cursor() as cursor:
            cursor.execute(f'''select count(*) from ckk_item_not_typed_top_level_view
                                where parent_id = %s or child_id = %s''', [id, id])
            qty, = cursor.fetchone()
            if qty == 0:
                cursor.execute(f'''select count(*) from ckk_item_typed_top_level_view
                                    where parent_id = %s or child_id = %s''', [id, id])
                qty, = cursor.fetchone()
            return qty > 0

    @staticmethod
    def delete_recursive(item_id, delete_lines=False, soft_delete=False, user=None, props=TurnBitOn(0, 0), document=None, show_progress=True):
        from isc_common.models.deleted_progresses import Deleted_progresses
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.ckk.models.item_line import Item_line
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.production.models.operations_item import Operations_item
        from kaf_pas.ckk.models.item_document import Item_document

        settings.LOCKS.acquire('ItemManager.delete_recursive')
        item_lotsman_document_cnt = 0
        if document != None:
            if ItemManager.check_4_production(item=item_id) == True:
                settings.LOCKS.release('ItemManager.delete_recursive')
                return 2

        sql_text = None
        if isinstance(document, Documents):
            sql_text = f'''WITH RECURSIVE r AS (
                                                    SELECT cir.*, 1 AS level
                                                    FROM ckk_item_refs cir
                                                             join ckk_item cich on cich.id = cir.child_id
                                                    WHERE cir.child_id IN ({item_id})
                                                      and cich.document_id = {document.id}
                                                
                                                    union all
                                                
                                                    SELECT cir.*, r.level + 1 AS level
                                                    FROM ckk_item_refs cir
                                                             join ckk_item cich on cich.id = cir.child_id
                                                             JOIN r ON cir.parent_id = r.child_id
                                                    WHERE cich.document_id = {document.id}
                                                )
                                                
                                                select *
                                                from r'''
        elif isinstance(document, Lotsman_documents_hierarcy):
            sql_text = f'''WITH RECURSIVE r AS (
                                                    SELECT cir.*, 1 AS level
                                                    FROM ckk_item_refs cir
                                                             join ckk_item cich on cich.id = cir.child_id
                                                    WHERE cir.child_id IN ({item_id})
                                                      and cich.lotsman_document_id = {document.id}

                                                    union all

                                                    SELECT cir.*, r.level + 1 AS level
                                                    FROM ckk_item_refs cir
                                                             join ckk_item cich on cich.id = cir.child_id
                                                             JOIN r ON cir.parent_id = r.child_id
                                                    WHERE cich.lotsman_document_id = {document.id}
                                                )

                                                select *
                                                from r'''

        cnt = Item_refs.objects.get_descendants_count(id=item_id, sql_text=sql_text)

        def _delete(progress=None):
            try:
                with transaction.atomic():
                    try:
                        item = Item.objects.get(id=item_id)
                        if cnt == 0:
                            Item_refs.objects1.filter(parent=item).delete_with_checked()
                            Item_line.objects.filter(parent=item).delete()
                            item.delete()
                        else:
                            _item = None
                            if progress != None:
                                progress.setContentsLabel(f'''<h3>Удаление товарной позиции ({item.STMP_1.value_str if item.STMP_1 else ''} : {item.STMP_2.value_str if item.STMP_2 else ''}).</h3>''')

                            for item in Item_refs.objects.get_descendants(id=item_id, order_by_clause='order by level desc', sql_text=sql_text):
                                _item = item
                                _continue = False
                                if not soft_delete:
                                    if document != None:
                                        if ItemManager.check_4_production(item=item.child_id) == True:
                                            _continue=True

                                    if _continue:
                                        _continue=False
                                        continue

                                    Item_image_refs.objects.filter(item_id=item.child_id).delete()

                                    if delete_lines:
                                        Item_line.objects.filter(parent_id=item.child_id).delete()
                                        Item_line.objects.filter(child_id=item.child_id).delete()

                                    Item_refs.objects1.filter(parent_id=item.child_id).delete_with_checked()
                                    Item_refs.objects1.filter(child_id=item.child_id).delete_with_checked()

                                    Operations_item.objects.filter(item=item.child_id).delete()
                                    # Ready_2_launch_detail.objects.filter(item=item.child_id).delete()

                                    Item_document.objects.filter(item_id=item.child_id).delete()
                                    Item.objects.filter(id=item.child_id).delete()
                                else:
                                    Item_image_refs.objects.filter(item_id=item.child_id).soft_delete()

                                    if delete_lines:
                                        Item_line.objects.filter(parent_id=item.child_id).soft_delete()
                                        Item_line.objects.filter(child_id=item.child_id).soft_delete()

                                        Item_line.objects.filter(parent_id=item.parent_id).soft_delete()
                                        Item_line.objects.filter(child_id=item.parent_id).soft_delete()

                                    Item_refs.objects.filter(parent_id=item.child_id).soft_delete()
                                    Item_refs.objects.filter(child_id=item.child_id).soft_delete()
                                    Item.objects.filter(id=item.child_id).soft_delete()

                                if progress != None:
                                    if  progress.step() != 0:
                                        raise ProgressDroped(progress_deleted)
                    except Item.DoesNotExist:
                        settings.LOCKS.release('ItemManager.delete_recursive')

            except db.models.deletion.ProtectedError as ex:
                logger.error(f'Item_refs: {item}')
                settings.LOCKS.release('ItemManager.delete_recursive')
                raise ex

            except ProgressDroped as ex:
                Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
                settings.LOCKS.release('ItemManager.delete_recursive')
                raise ex

        if show_progress and cnt > 0:
            with managed_progress(
                    id=f'delete_recursive_{item_id}',
                    qty=cnt,
                    user=user,
                    message='Удаление товарной(ых) позиций',
                    title='Выполнено',
                    props=props
            ) as progress:
               _delete(progress)
        else:
            _delete()

        settings.LOCKS.release('ItemManager.delete_recursive')

        if item_lotsman_document_cnt > 0:
            refresh_mat_view('kd_lotsman_documents_hierarcy_mview')

    def updateFromRequest(self, request, removed=None, function=None):
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.ckk.models.item_line import Item_line

        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        with transaction.atomic():
            targetRecord = data.get('targetRecord')
            dropRecords = data.get('dropRecords')
            if data.get('mode') == 'move':
                res = 0
                for dropRecord in dropRecords:
                    res += Item_refs.objects.filter(parent_id=dropRecord.get('parent_id'), child_id=dropRecord.get('id')).update(parent_id=targetRecord.get('id'))
                    res += Item_line.objects.filter(parent_id=dropRecord.get('parent_id'), child_id=dropRecord.get('id')).update(parent_id=targetRecord.get('id'))
                return res
            elif data.get('mode') == 'copy':
                res = 0
                for dropRecord in dropRecords:
                    Item_refs.objects.create(parent_id=targetRecord.get('id'), child_id=dropRecord.get('id'))
                    res += 1
                return res
            elif data.get('mode') == 'replace':
                res = 0
                for dropRecord in dropRecords:
                    Item_refs.objects.create(parent_id=targetRecord.get('parent_id'), child_id=dropRecord.get('id'))
                    res += 1

                    item_line = model_to_dict(Item_line.objects.get(parent_id=targetRecord.get('parent_id'), child_id=targetRecord.get('id')))
                    delAttr(item_line, 'id')
                    delAttr(item_line, 'parent')
                    delAttr(item_line, 'child')
                    setAttr(item_line, 'parent_id', targetRecord.get('parent_id'))
                    setAttr(item_line, 'child_id', dropRecord.get('id'))
                    setAttr(item_line, 'SPC_CLM_NAME_id', dropRecord.get('STMP_1_id'))
                    delAttr(item_line, 'SPC_CLM_NAME')
                    setAttr(item_line, 'SPC_CLM_MARK_id', dropRecord.get('STMP_2_id'))
                    delAttr(item_line, 'SPC_CLM_MARK')
                    setAttr(item_line, 'SPC_CLM_FORMAT_id', item_line.get('SPC_CLM_FORMAT'))
                    delAttr(item_line, 'SPC_CLM_FORMAT')
                    setAttr(item_line, 'SPC_CLM_ZONE_id', item_line.get('SPC_CLM_ZONE'))
                    delAttr(item_line, 'SPC_CLM_ZONE')
                    setAttr(item_line, 'SPC_CLM_POS_id', item_line.get('SPC_CLM_POS'))
                    delAttr(item_line, 'SPC_CLM_POS')
                    setAttr(item_line, 'SPC_CLM_COUNT_id', item_line.get('SPC_CLM_COUNT'))
                    delAttr(item_line, 'SPC_CLM_COUNT')
                    setAttr(item_line, 'SPC_CLM_NOTE_id', item_line.get('SPC_CLM_NOTE'))
                    delAttr(item_line, 'SPC_CLM_NOTE')
                    setAttr(item_line, 'SPC_CLM_MASSA_id', item_line.get('SPC_CLM_MASSA'))
                    delAttr(item_line, 'SPC_CLM_MASSA')
                    setAttr(item_line, 'SPC_CLM_MATERIAL_id', item_line.get('SPC_CLM_MATERIAL'))
                    delAttr(item_line, 'SPC_CLM_MATERIAL')
                    setAttr(item_line, 'SPC_CLM_USER_id', item_line.get('SPC_CLM_USER'))
                    delAttr(item_line, 'SPC_CLM_USER')
                    setAttr(item_line, 'SPC_CLM_KOD_id', item_line.get('SPC_CLM_KOD'))
                    delAttr(item_line, 'SPC_CLM_KOD')
                    setAttr(item_line, 'SPC_CLM_FACTORY_id', item_line.get('SPC_CLM_FACTORY'))
                    delAttr(item_line, 'SPC_CLM_FACTORY')
                    Item_line.objects.create(**item_line)
                    res += 1

                    Item_refs.objects.filter(parent_id=targetRecord.get('parent_id'), child_id=targetRecord.get('id')).delete()
                    Item_line.objects.filter(parent_id=targetRecord.get('parent_id'), child_id=targetRecord.get('id')).delete()

                    try:
                        Item.objects.filter(id=targetRecord.get('id')).delete()
                    except Exception as ex:
                        logger.debug(ex)

                return res

            else:
                def rec(data):
                    _data = data.copy()
                    relevant = data.get('relevant')
                    confirmed = data.get('confirmed')
                    props = data.get('props')

                    if relevant == 'Актуален':
                        props |= Item.props.relevant
                    else:
                        props &= ~Item.props.relevant

                    if confirmed == 'Подтвержден':
                        props |= Item.props.confirmed
                    else:
                        props &= ~Item.props.confirmed

                    where_from = data.get('where_from')
                    if where_from == 'Получено из чертежа':
                        props |= Item.props.from_cdw
                    elif where_from == 'Получено из спецификации':
                        props |= Item.props.from_spw
                    elif where_from == 'Получено из бумажного архива':
                        props |= Item.props.from_pdf
                    elif where_from == 'Занесено вручную':
                        props |= Item.props.man_input

                    delAttr(data, 'relevant')
                    delAttr(data, 'confirmed')
                    delAttr(data, 'where_from')
                    delAttr(data, 'qty_operations')
                    delAttr(data, 'refs_props')
                    delAttr(data, 'icon')
                    delAttr(data, 'id')
                    delAttr(data, 'STMP_1__value_str')
                    delAttr(data, 'STMP_2__value_str')
                    delAttr(data, 'document__file_document')
                    delAttr(data, 'isFolder')
                    delAttr(data, 'isLotsman')
                    delAttr(data, 'parent_id')
                    delAttr(data, 'section')

                    setAttr(data, 'props', props)

                    res = Item.objects.filter(id=_data.get('id')).update(**data)

                    return _data

                if isinstance(data, dict):
                    for k, _data in data.items():
                        if isinstance(_data, dict):
                            res = rec(_data)
                        else:
                            res = rec(data.copy())
                            break
                else:
                    res = 0

            return res

    def deleteFromRequest(self, request, removed=None, ):
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.ckk.models.item_line import Item_line

        request = DSRequest(request=request)
        data = request.get_data()
        res = 0

        mode = None
        with transaction.atomic():
            operations = get_operations_from_trunsaction(data)
            if isinstance(operations, list):
                for operation in get_operations_from_trunsaction(data):
                    settings.LOCKS.acquire('ItemManager.deleteFromRequest')
                    _data = operation.get('data')
                    if _data.get('mode') == 'deleteInner':
                        mode = 'deleteInner'
                        records = _data.get('records')

                        if isinstance(records, list):
                            for record in records:
                                res, _ = Item_refs.objects1.filter(child_id=record.get('id'), parent_id=record.get('parent_id')).delete_with_checked()
                    settings.LOCKS.release('ItemManager.deleteFromRequest')

                if mode == 'deleteInner':
                    return res
            elif isinstance(data, dict):
                if data.get('mode') == 'deleteInner':
                    settings.LOCKS.acquire('ItemManager.deleteFromRequest')
                    records = data.get('records')

                    if isinstance(records, list):
                        for record in records:
                            res, _ = Item_refs.objects1.filter(child_id=record.get('id'), parent_id=record.get('parent_id')).delete_with_checked()
                        settings.LOCKS.release('ItemManager.deleteFromRequest')
                        return res
                    settings.LOCKS.release('ItemManager.deleteFromRequest')
                elif data.get('mode') == 'reloadRefs':
                    records = data.get('records')
                    res = 0
                    if isinstance(records, list):
                        settings.LOCKS.acquire('ItemManager.deleteFromRequest')
                        for record in records:
                            id = record.get('id')
                            parent_id = record.get('parent_id')

                            for item_line in Item_line.objects.filter(parent_id=id):
                                item_refs, _ = Item_refs.objects.get_or_create(parent_id=id, child=item_line.child)
                                res += 1
                        settings.LOCKS.release('ItemManager.deleteFromRequest')

                    return res

        for id, mode in request.get_tuple_ids():
            if mode == 'hide':
                res = self.delete_recursive(item_id=id, soft_delete=True, delete_lines=True, user=request.user_id)
            else:
                res = self.delete_recursive(item_id=id, soft_delete=False, delete_lines=True, user=request.user_id)
        return res

    def lookupFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()

        attr_type__code = data.get('attr_type__code')
        value_str = data.get(f'{attr_type__code}__value_str')
        value_int = data.get(f'{attr_type__code}__value_int')

        if value_str == None and value_int != None:
            value_str = str(value_int)

        if value_str == None:
            return None

        res, created = Document_attributesManager.get_or_create_attribute(attr_codes=attr_type__code, value_str=value_str)
        res = model_to_dict(res)
        if res.get('value_str') == 'null':
            setAttr(res, 'value_str', None)
        return res

    @staticmethod
    def get_verstion(STMP_1, STMP_2, props, inc=1):
        settings.LOCKS.acquire('ItemManager.get_verstion')
        try:
            res = Item.objects.filter(
                STMP_1__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_1.value_str) if STMP_1 else None,
                STMP_2__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_2.value_str) if STMP_2 else None,
                props=props,
                version__isnull=False
            ).order_by('-version')[0].version + inc
            if res == None:
                res = 1
        except IndexError:
            res = None
        settings.LOCKS.release('ItemManager.get_verstion')
        return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'STMP_1_id': record.STMP_1.id if record.STMP_1 else None,
            'STMP_1__value_str': record.STMP_1.value_str if record.STMP_1 else None,
            'STMP_2_id': record.STMP_2.id if record.STMP_2 else None,
            'STMP_2__value_str': record.STMP_2.value_str if record.STMP_2 else None,
            'lastmodified': record.lastmodified,
            'version': record.version,
            'document_id': record.document.id if record.document else None,
            'document__file_document': record.document.file_document if record.document else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return ItemQuerySet(self.model, using=self._db)

    def checkRecursives(self, request):
        from kaf_pas.ckk.management.commands.check_item_cicular_refs import check_level
        from isc_common.models.deleted_progresses import Deleted_progresses

        request = DSRequest(request=request)
        data = request.get_data()
        if isinstance(data, dict) and isinstance(data.get('records'), list):
            cycle_ref = []
            with managed_progress(
                    id=f'''checkRecursives_{request.user.id}''',
                    qty=1000,
                    user=request.user,
                    message='<h3>Поиск циклических зависемостей.</h3>',
                    title='Выполнено',
                    props=TurnBitOn(0, 0)
            ) as progress:
                try:
                    for record in data.get('records'):
                        cycle_ref.extend(check_level(parent_array=list(), item=Item.objects.get(id=record.get('id')), level=1, table='Item_refs', progress=progress))
                    if len(cycle_ref) == 0:
                        progress.sendInfo('Циклических зависемостей не найдено.')
                    else:
                        progress.sendInfo('\n'.join(cycle_ref))
                    return dict(messages=cycle_ref)
                except ProgressDroped as ex:
                    Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
                    raise ex
        raise Exception(f'Not data.')

    @staticmethod
    def _getQtyChilds(records):
        from kaf_pas.ckk.models.item_qty import Item_qty

        res = 0
        if isinstance(records, list):
            with connection.cursor() as cursor:
                for record in records:
                    cursor.execute(f'''WITH RECURSIVE r AS (
                                                                SELECT *, 1 AS level
                                                                FROM ckk_item_refs
                                                                WHERE parent_id IN (%s)
    
                                                                union all
    
                                                                SELECT ckk_item_refs.*, r.level + 1 AS level
                                                                FROM ckk_item_refs
                                                                         JOIN r
                                                                              ON ckk_item_refs.parent_id = r.child_id)
    
                                                            select  count(*)
                                                            from r''', [record.get('id')])
                    qty, = cursor.fetchone()
                    res += qty
                    Item_qty.objects.create(
                        child_id=record.get('id'),
                        parent_id=record.get('parent_id'),
                        qty=qty
                    )
        return dict(qty=res)

    def getQtyChilds(self, request):

        request = DSRequest(request=request)
        data = request.get_data()
        if isinstance(data, dict):
            records = data.get('records')
            if isinstance(records, list):
                return ItemManager._getQtyChilds(records=records)


class Item_add:
    @staticmethod
    def get_prop_field():
        return BitField(flags=(
            ('relevant', 'Актуальность'),  # 1
            ('from_cdw', 'Получено из чертежа'),  # 2
            ('from_spw', 'Получено из спецификации'),  # 4
            ('for_line', 'Строка спецификации'),  # 8
            ('from_pdf', 'Получено из бумажного архива'),  # 16
            ('man_input', 'Занесено вручную'),  # 32
            ('copied', 'Скопировано'),  # 64
            ('from_lotsman', 'Получено из Лоцмана'),  # 128
            ('confirmed', 'Подтвержден'),  # 256
        ), default=1, db_index=True)


class Item(AuditModel):
    STMP_1 = ForeignKeyProtect(Document_attributes, verbose_name='Наименование изделия', related_name='STMP_1', null=True, blank=True)
    STMP_2 = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение изделия', related_name='STMP_2', null=True, blank=True)
    version = PositiveIntegerField(null=True, blank=True)

    props = Item_add.get_prop_field()

    document = ForeignKeyProtect(Documents, verbose_name='Документ', null=True, blank=True)
    lotsman_document = ForeignKeyProtect(Lotsman_documents_hierarcy, verbose_name='Документ из Лоцмана', null=True, blank=True)
    lotsman_type_id = BigIntegerField(null=True, blank=True)

    objects = ItemManager()

    @property
    def item_name(self):
        if self.STMP_1 != None and self.STMP_2 != None:
            return f'{get_NoneStr(self.STMP_1.value_str)}: {get_NoneStr(self.STMP_2.value_str)}'
        elif self.STMP_1 == None and self.STMP_2 != None:
            return get_NoneStr(self.STMP_2.value_str)
        elif self.STMP_1 != None and self.STMP_2 == None:
            return get_NoneStr(self.STMP_1.value_str)
        else:
            return 'Неизвестен'

    @staticmethod
    def get_vaslue_str(doc_attr):
        if doc_attr.value_str == None:
            return None
        return doc_attr.value_str.strip() if doc_attr else None

    def __str__(self):
        return f'ID={self.id} STMP_1=[{self.STMP_1}], STMP_2=[{self.STMP_2}], props={self.props}, version={self.version}'

    def __repr__(self):
        return str(self.id)

    class Meta:
        verbose_name = 'Товарная позиция'
        constraints = [
            UniqueConstraint(fields=['props'], condition=Q(STMP_1=None) & Q(STMP_2=None) & Q(lotsman_document_id=None) & Q(lotsman_type_id=None) & Q(version=None), name='Item_unique_constraint_0'),
            UniqueConstraint(fields=['STMP_1', 'props'], condition=Q(STMP_2=None) & Q(lotsman_document_id=None) & Q(lotsman_type_id=None) & Q(version=None), name='Item_unique_constraint_1'),
            UniqueConstraint(fields=['STMP_2', 'props'], condition=Q(STMP_1=None) & Q(lotsman_document_id=None) & Q(lotsman_type_id=None) & Q(version=None), name='Item_unique_constraint_2'),
            UniqueConstraint(fields=['STMP_1', 'STMP_2', 'props'], condition=Q(lotsman_document_id=None) & Q(lotsman_type_id=None) & Q(version=None), name='Item_unique_constraint_3'),
            UniqueConstraint(fields=['props', 'version'], condition=Q(STMP_1=None) & Q(STMP_2=None) & Q(lotsman_document_id=None) & Q(lotsman_type_id=None), name='Item_unique_constraint_4'),
            UniqueConstraint(fields=['STMP_1', 'props', 'version'], condition=Q(STMP_2=None) & Q(lotsman_document_id=None) & Q(lotsman_type_id=None), name='Item_unique_constraint_5'),
            UniqueConstraint(fields=['STMP_2', 'props', 'version'], condition=Q(STMP_1=None) & Q(lotsman_document_id=None) & Q(lotsman_type_id=None), name='Item_unique_constraint_6'),
            UniqueConstraint(fields=['STMP_1', 'STMP_2', 'props', 'version'], condition=Q(lotsman_document_id=None) & Q(lotsman_type_id=None), name='Item_unique_constraint_7'),
            UniqueConstraint(fields=['lotsman_document_id', 'props'], condition=Q(STMP_1=None) & Q(STMP_2=None) & Q(lotsman_type_id=None) & Q(version=None), name='Item_unique_constraint_8'),
            UniqueConstraint(fields=['STMP_1', 'lotsman_document_id', 'props'], condition=Q(STMP_2=None) & Q(lotsman_type_id=None) & Q(version=None), name='Item_unique_constraint_9'),
            UniqueConstraint(fields=['STMP_2', 'lotsman_document_id', 'props'], condition=Q(STMP_1=None) & Q(lotsman_type_id=None) & Q(version=None), name='Item_unique_constraint_10'),
            UniqueConstraint(fields=['STMP_1', 'STMP_2', 'lotsman_document_id', 'props'], condition=Q(lotsman_type_id=None) & Q(version=None), name='Item_unique_constraint_11'),
            UniqueConstraint(fields=['lotsman_document_id', 'props', 'version'], condition=Q(STMP_1=None) & Q(STMP_2=None) & Q(lotsman_type_id=None), name='Item_unique_constraint_12'),
            UniqueConstraint(fields=['STMP_1', 'lotsman_document_id', 'props', 'version'], condition=Q(STMP_2=None) & Q(lotsman_type_id=None), name='Item_unique_constraint_13'),
            UniqueConstraint(fields=['STMP_2', 'lotsman_document_id', 'props', 'version'], condition=Q(STMP_1=None) & Q(lotsman_type_id=None), name='Item_unique_constraint_14'),
            UniqueConstraint(fields=['STMP_1', 'STMP_2', 'lotsman_document_id', 'props', 'version'], condition=Q(lotsman_type_id=None), name='Item_unique_constraint_15'),
            UniqueConstraint(fields=['lotsman_type_id', 'props'], condition=Q(STMP_1=None) & Q(STMP_2=None) & Q(lotsman_document_id=None) & Q(version=None), name='Item_unique_constraint_16'),
            UniqueConstraint(fields=['STMP_1', 'lotsman_type_id', 'props'], condition=Q(STMP_2=None) & Q(lotsman_document_id=None) & Q(version=None), name='Item_unique_constraint_17'),
            UniqueConstraint(fields=['STMP_2', 'lotsman_type_id', 'props'], condition=Q(STMP_1=None) & Q(lotsman_document_id=None) & Q(version=None), name='Item_unique_constraint_18'),
            UniqueConstraint(fields=['STMP_1', 'STMP_2', 'lotsman_type_id', 'props'], condition=Q(lotsman_document_id=None) & Q(version=None), name='Item_unique_constraint_19'),
            UniqueConstraint(fields=['lotsman_type_id', 'props', 'version'], condition=Q(STMP_1=None) & Q(STMP_2=None) & Q(lotsman_document_id=None), name='Item_unique_constraint_20'),
            UniqueConstraint(fields=['STMP_1', 'lotsman_type_id', 'props', 'version'], condition=Q(STMP_2=None) & Q(lotsman_document_id=None), name='Item_unique_constraint_21'),
            UniqueConstraint(fields=['STMP_2', 'lotsman_type_id', 'props', 'version'], condition=Q(STMP_1=None) & Q(lotsman_document_id=None), name='Item_unique_constraint_22'),
            UniqueConstraint(fields=['STMP_1', 'STMP_2', 'lotsman_type_id', 'props', 'version'], condition=Q(lotsman_document_id=None), name='Item_unique_constraint_23'),
            UniqueConstraint(fields=['lotsman_document_id', 'lotsman_type_id', 'props'], condition=Q(STMP_1=None) & Q(STMP_2=None) & Q(version=None), name='Item_unique_constraint_24'),
            UniqueConstraint(fields=['STMP_1', 'lotsman_document_id', 'lotsman_type_id', 'props'], condition=Q(STMP_2=None) & Q(version=None), name='Item_unique_constraint_25'),
            UniqueConstraint(fields=['STMP_2', 'lotsman_document_id', 'lotsman_type_id', 'props'], condition=Q(STMP_1=None) & Q(version=None), name='Item_unique_constraint_26'),
            UniqueConstraint(fields=['STMP_1', 'STMP_2', 'lotsman_document_id', 'lotsman_type_id', 'props'], condition=Q(version=None), name='Item_unique_constraint_27'),
            UniqueConstraint(fields=['lotsman_document_id', 'lotsman_type_id', 'props', 'version'], condition=Q(STMP_1=None) & Q(STMP_2=None), name='Item_unique_constraint_28'),
            UniqueConstraint(fields=['STMP_1', 'lotsman_document_id', 'lotsman_type_id', 'props', 'version'], condition=Q(STMP_2=None), name='Item_unique_constraint_29'),
            UniqueConstraint(fields=['STMP_2', 'lotsman_document_id', 'lotsman_type_id', 'props', 'version'], condition=Q(STMP_1=None), name='Item_unique_constraint_30'),
            UniqueConstraint(fields=['STMP_1', 'STMP_2', 'lotsman_document_id', 'lotsman_type_id', 'props', 'version'], name='Item_unique_constraint_31'),
        ]
        db_constraints = {
            'Item_not_null_STMP1_STMP2': 'CHECK ("STMP_1_id" IS NOT NULL OR "STMP_2_id" IS NOT NULL)',
        }



# Generated by Django 2.2 on 2019-04-16 10:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0021_auto_20190415_1307'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item',
            unique_together={('STMP_1', 'STMP_2', 'props')},
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-04 11:17

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0070_auto_20190829_1017'),
        ('ckk', '0161_auto_20191122_0920'),
    ]

    operations = [
        migrations.AddField(
            model_name='locations',
            name='calendar',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='clndr.Calendars'),
        ),
    ]



import logging

from bitfield import BitField
from django.conf import settings
from django.db import transaction

from isc_common import setAttr, delAttr
from isc_common.bit import TurnBitOn, TurnBitOff
from isc_common.common import blinkString
from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefManager, BaseRefQuerySet, BaseRef
from isc_common.number import DelProps
from kaf_pas.planing.models.operation_types import Operation_types

logger = logging.getLogger(__name__)


class Status_operation_typesQuerySet(BaseRefQuerySet):
    pass


class Status_operation_typesManager(BaseRefManager):

    def updateFromRequest(self, request, removed=None, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        setAttr(_data, 'opertype_id', _data.get('opertype'))
        delAttr(_data, 'opertype')

        disabled = _data.get('disabled')
        blink = _data.get('blink')
        props = _data.get('props', 0)

        delAttr(_data, 'disabled')
        delAttr(_data, 'blink')
        props = TurnBitOn(props, 0) if disabled else TurnBitOff(props, 0)
        props = TurnBitOn(props, 1) if blink else TurnBitOff(props, 1)
        setAttr(_data, 'props', props)

        super().filter(id=_data.get('id')).update(**_data)
        setAttr(data, 'props', props)
        return data

    @staticmethod
    def make_statuses(opertype, status_map):
        res = dict()
        settings.LOCKS.acquire('Status_operation_typesManager.make_statuses')
        with transaction.atomic():
            for key_value in status_map:
                status, _ = Status_operation_types.objects.update_or_create(
                    opertype=opertype,
                    code=key_value.get('code'),
                    defaults=dict(
                        props=key_value.get('props'),
                        name=key_value.get('name'),
                        color=key_value.get('color'),
                        deliting=False,
                        editing=False,
                    ))
                res.setdefault(key_value.get('code'), status)
        settings.LOCKS.release('Status_operation_typesManager.make_statuses')
        return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'editing': record.editing,
            'color': record.color,
            'deliting': record.deliting,
            'disabled': record.props.disabled,
            'blink': record.props.blink,
            'props': record.props,
            'opertype': record.opertype.id,
        }
        return DelProps(res)

    def get_queryset(self):
        return Status_operation_typesQuerySet(self.model, using=self._db)


class Status_operation_types(BaseRef):
    opertype = ForeignKeyCascade(Operation_types)
    color = CodeField(null=True, blank=True)

    props = BitField(flags=(
        ('disabled', 'Неактивная запись в гриде'),
        ('blink', 'Мигающее поле в гриде')
    ), default=0, db_index=True)

    objects = Status_operation_typesManager()

    @property
    def wrap_name(self):
        return blinkString(text=self.name, color=self.color, blink=self.props.blink)

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}, opertype: [{self.opertype}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Статусы запусков'



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_flat_view import Item_flat_view, Item_flat_viewManager


@JsonResponseWithException()
def Item_flat_view_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Item_flat_view.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Item_flat_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_flat_view_FetchPlan(request):
    from isc_common.http.DSRequest import DSRequest
    from kaf_pas.planing.models.operations_view import Operations_view

    _request = DSRequest(request=request)
    data = _request.get_data()

    item_ids = [item.get('item_id') for item in Operations_view.objects.filter(
        launch_id=data.get('launch_id'),
        operation_level_id=data.get('level_id'),
        resource_id=data.get('resource_id'),
        resource__location_id=data.get('location_id'),
    ).values('item_id').distinct()]

    return JsonResponse(
        DSResponse(
            request=request,
            data=Item_flat_view.objects.
                filter(id__in=item_ids).
                get_range_rows1(
                request=request,
                function=Item_flat_viewManager.getRecord,
                remove_fields=['launch_id', 'level_id', 'resource_id', 'location_id']
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_flat_view_Add(request):
    return JsonResponse(DSResponseAdd(data=Item.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_flat_view_Update(request):
    return JsonResponse(DSResponseUpdate(data=Item.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_flat_view_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Item.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_flat_view_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Item.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_flat_view_Info(request):
    return JsonResponse(DSResponse(request=request, data=Item_flat_view.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_flat_view_InfoPlan(request):
    return JsonResponse(DSResponse(request=request, data=Item_flat_view.objects.get_queryset().get_infoPlan(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Item_flat_view_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Item_flat_view.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.0.3 on 2020-03-02 03:21

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0141_auto_20200222_1755'),
        ('production', '0224_auto_20200229_1523'),
    ]

    operations = [
        migrations.AddField(
            model_name='launch_item_line',
            name='SPC_CLM_COUNT_PER_ONE',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Launch_SPC_CLM_COUNT_PER_ONE', to='kd.Document_attributes', verbose_name='Количество на еденицу'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-26 10:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0209_auto_20200226_0828'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_operation_resources',
            name='batch_size',
            field=models.PositiveIntegerField(db_index=True, default=1),
        ),
        migrations.AlterField(
            model_name='launch_operations_item',
            name='num',
            field=models.PositiveIntegerField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='launch_operations_item',
            name='qty',
            field=models.PositiveIntegerField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='launch_operations_material',
            name='qty',
            field=models.FloatField(db_index=True),
        ),
    ]



import logging

from django.db.models import Model, PositiveIntegerField, Manager, QuerySet, Max

from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import Documents

logger = logging.getLogger(__name__)


class Document_attr_cross1QuerySet(QuerySet):
    def delete(self):
        return super().delete()

    def _position_in_document(self, **kwargs):
        position_in_document = kwargs.get('position_in_document')
        document = kwargs.get('document')
        if document != None:
            if position_in_document == None:
                position_in_document_dict = Document_attr_cross.objects.filter(document=document).aggregate(Max('position_in_document'))
                position_in_document = position_in_document_dict.get('position_in_document__max')
                if position_in_document == None:
                    position_in_document = 1
                else:
                    position_in_document += 1
                kwargs.setdefault('position_in_document', position_in_document)
                return kwargs
            else:
                return kwargs
        else:
            raise Exception(f'Not have a document paremetr.')

    def create(self, **kwargs):
        return super().create(**self._position_in_document(**kwargs))

    # def update(self, **kwargs):
    #     return super().create(**self._position_in_document(**kwargs))

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Document_attr_cross1Manager(Manager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Document_attr_cross1QuerySet(self.model, using=self._db)


class Document_attr_cross(Model):
    document = ForeignKeyCascade(Documents)
    attribute = ForeignKeyProtect(Document_attributes)
    position_in_document = PositiveIntegerField()

    section = CodeField(verbose_name="Раздел", null=True, blank=True)
    subsection = CodeField(verbose_name="Подраздел", null=True, blank=True)

    objects = Document_attr_cross1Manager()

    # def save(self, *args, **kwargs):
    #     if self.attribute.attr_type.code == 'STMP_1' or self.attribute.attr_type.code == 'STMP_2':
    #         try:
    #             Document_attr_cross.objects.get(attribute=self.attribute, document=self.document)
    #             raise Exception(f'Попытка записи дубликата: {self.attribute}')
    #         except Document_attr_cross.DoesNotExist:
    #             ...
    #         except Document_attr_cross.MultipleObjectsReturned:
    #             ...
    #  super().save(*args, **kwargs)

    def __str__(self):
        return f"{self.id}, document: [{self.document}] attribute: [{self.attribute}], position_in_document: {self.position_in_document}, section: {self.section}, subsection: {self.subsection}"

    class Meta:
        verbose_name = 'Кросс таблица'
        unique_together = (('document', 'position_in_document'),)



# Generated by Django 2.2 on 2019-04-21 13:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0021_documents_view'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='pathes',
            table='pathes',
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-28 11:26

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0051_documents_load_error'),
    ]

    operations = [
        migrations.AddField(
            model_name='pathes',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('ignored_on_load', 'Не пересматривать обновление контента')), db_index=True, default=1),
        ),
    ]



import logging

from bitfield import BitField
from django.db.models import DecimalField
from django.forms import model_to_dict
from isc_common import setAttr, delAttr
from isc_common.bit import IsBitOn
from isc_common.fields.code_field import CodeStrictField
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.base_ref import BaseRefHierarcy
from isc_common.number import DelProps, GetPropsInt

logger = logging.getLogger(__name__)


class Ed_izmQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Ed_izmManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'koef_recalc': record.koef_recalc,
            'description': record.description,
            'parent_id': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
            'props': GetPropsInt(record.props),
            'root_timevalue': IsBitOn(record.props, 0)
        }
        return res

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        props = 0
        if _data.get('root_timevalue') != None and isinstance(_data.get('root_timevalue'), bool):
            if _data.get('root_timevalue'):
                props |= Ed_izm.props.root_timevalue
            else:
                props &= ~Ed_izm.props.root_timevalue

            delAttr(_data, 'root_timevalue')
            setAttr(_data, 'props', props)

        res = super().create(**_data)
        try:
            full_name = res.full_name
        except:
            full_name = None
        res = model_to_dict(res)
        if full_name:
            setAttr(res, 'full_name', full_name)

        setAttr(res, 'isFolder', False)
        data.update(DelProps(res))
        return data

    def updateFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        _data = data.copy()
        _data1 = _data.copy()

        props = DelProps(_data).get('props')

        if _data.get('root_timevalue') != None and isinstance(_data.get('root_timevalue'), bool):
            if _data.get('root_timevalue'):
                props |= Ed_izm.props.root_timevalue
            else:
                props &= ~Ed_izm.props.root_timevalue

            delAttr(_data, 'root_timevalue')
            setAttr(_data, 'props', props)

        super().filter(id=request.get_id()).update(**_data)
        return _data1

    def get_queryset(self):
        return Ed_izmQuerySet(self.model, using=self._db)


class Ed_izm(BaseRefHierarcy):
    code = CodeStrictField(unique=True)
    koef_recalc = DecimalField(max_digits=19, decimal_places=4, null=True, blank=True)

    props = BitField(flags=(
        ('root_timevalue', 'Базовая временная единица'),  # 1
    ), default=0, db_index=True)

    objects = Ed_izmManager()

    @staticmethod
    def root_timevalue():
        return Ed_izm.objects.get(props=Ed_izm.props.root_timevalue)

    def __str__(self):
        return f"ID: {self.id}, code: {self.code}, koef_recalc: {self.koef_recalc}"

    class Meta:
        verbose_name = 'Единицы измерения'



import logging

from django.db.models import CharField
from isc_common.fields.code_field import CodeField
from isc_common.models.base_ref import BaseRefManager, BaseRefQuerySet, BaseRefHierarcy

logger = logging.getLogger(__name__)


class Material_typeQuerySet(BaseRefQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Material_typeManager(BaseRefManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'full_name': record.full_name,
            'gost': record.gost,
            'description': record.description,
            'parent_id': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Material_typeQuerySet(self.model, using=self._db)


class Material_type(BaseRefHierarcy):
    code = CharField(max_length=255, unique=True)
    gost = CodeField()
    name = CharField(max_length=255)
    objects = Material_typeManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Тип материала'



# Generated by Django 2.2.1 on 2019-05-21 19:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0052_merge_20190521_1855'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='status',
            unique_together={('code_owner', 'code')},
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-04 16:07

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0141_resource_users'),
    ]

    operations = [
        migrations.AddField(
            model_name='resource_users',
            name='props',
            field=bitfield.models.BitField((('compulsory_reading', 'Обязательное прочтение'),), db_index=True, default=1),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-13 16:07

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0099_auto_20190613_1607'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_line',
            name='child',
            field=isc_common.fields.related.ForeignKeyCascade(default=None, on_delete=django.db.models.deletion.CASCADE, related_name='item_child', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-28 15:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0026_auto_20190524_1200'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Operation_plan_template_locations',
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-30 13:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0046_operations_parent'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operations',
            name='end',
        ),
        migrations.RemoveField(
            model_name='operations',
            name='start',
        ),
    ]



from enum import Enum, IntEnum


class constants1:
    documentation = 5
    complex = 10
    assembly_units = 15
    minutiae = 20
    standard_products = 25
    other_products = 30
    materials = 35
    kits = 40

    # spcObjType - тип объектов:
    # 0 - базовые,
    # 1 - вспомогательные,
    # 2 - базовые и вспомогательные из сортирован­ного массива,
    # 3 - все объекты.

    spcObjTypeBase = 0
    spcObjTypeAuxiliary = 1
    spcObjTypeBaseAuxiliarySorted = 2
    spcObjTypeAll = 3

    # ksMoveIterator - направление перемещения итератора:
    ksMoveIteratorF = 'F'  # - первый объект,
    ksMoveIteratorN = 'N'  # - следующий объект.

    # additioanalCol - признак колонок:

    additioanalColTable = 0  # - колонки таблицы спецификации,
    additioanalColAdded = 1  # - дополнительныеколонки


class specRow:
    format = None
    zone = None
    pos = None
    mark = None
    name = None
    count = None
    note = None
    massa = None
    material = None
    user = None
    code = None
    factory = None
    complex = None


class ColumnType(IntEnum):
    SPC_CLM_FORMAT = 1  # Формат
    SPC_CLM_ZONE = 2  # Зона
    SPC_CLM_POS = 3  # Позиция
    SPC_CLM_MARK = 4  # Обозначение
    SPC_CLM_NAME = 5  # Наименование
    SPC_CLM_COUNT = 6  # Количество
    SPC_CLM_NOTE = 7  # Примечание
    SPC_CLM_MASSA = 8  # Масса
    SPC_CLM_MATERIAL = 9  # Материал
    SPC_CLM_USER = 10  # Пользовательская
    SPC_CLM_KOD = 11  # Код
    SPC_CLM_FACTORY = 12  # Предприятие - изготовитель


class FormatsRastr(IntEnum):
    FORMAT_BMP = 0 #- BMP,
    FORMAT_GIF = 1 #- GIF,
    FORMAT_JPG = 2 #- JPEG,
    FORMAT_PNG = 3 #- PNG,
    FORMAT_TIF = 4 #- TIFF,
    FORMAT_TGA = 5 #- TGA,
    FORMAT_PCX = 6 #- PCX
    #FORMAT_WMF = 16 #- WMF(неподдерживается)
    FORMAT_EMF = 17 #- EMF

class DeepOfColer(IntEnum):
    BPP_COLOR_01 = 1 #- монохромный,
    BPP_COLOR_02 = 2 #- 4 цвета,
    BPP_COLOR_04 = 4 #- 16 цветов,
    BPP_COLOR_08 = 8 #- 256 цветов,
    BPP_COLOR_16 = 16 #- 16 разрядов,
    BPP_COLOR_24 = 24 #- 24 разряда,
    BPP_COLOR_32 = 32 #- 32 разряда.

class Color4OutObj(IntEnum):
    BLACKWHITE = 0 #- черный,
    COLORVIEW = 1 #- цвет, установленный для вида,
    COLORLAYER =  2 #- цвет, установленный для слоя,
    COLOROBJECT = 3 #- цвет, установленный для объекта.









# Generated by Django 3.0.3 on 2020-02-17 12:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0188_remove_resource_parent'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='launch_item_refs',
            name='added_launch',
        ),
        migrations.RemoveField(
            model_name='launch_item_refs',
            name='description',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-21 06:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0057_auto_20190820_1858'),
    ]

    operations = [
        migrations.RenameField(
            model_name='operation_material',
            old_name='material',
            new_name='material_askon',
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-23 15:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0203_launch_item_refs_qty'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_refs',
            name='qty',
            field=models.DecimalField(blank=True, decimal_places=4, default=0.0, max_digits=19, null=True),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-21 04:46

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0181_auto_20191217_1331'),
        ('kd', '0122_remove_lotsman_document_attr_cross_position_in_document'),
    ]

    operations = [
        migrations.AddField(
            model_name='pathes',
            name='attr_type',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Attr_type', verbose_name='Атрибут'),
        ),
    ]



from django.urls import path

from kaf_pas.ckk.views import locations_users

urlpatterns = [

    path('Locations_users/Fetch/', locations_users.Locations_users_Fetch),
    path('Locations_users/Add', locations_users.Locations_users_Add),
    path('Locations_users/Update', locations_users.Locations_users_Update),
    path('Locations_users/CopyUsers', locations_users.Locations_users_CopyUsers),
    path('Locations_users/Remove', locations_users.Locations_users_Remove),
    path('Locations_users/Lookup/', locations_users.Locations_users_Lookup),
    path('Locations_users/Info/', locations_users.Locations_users_Info),
    path('Locations_users/Copy', locations_users.Locations_users_Copy),

]



from django.urls import path

from kaf_pas.kd.views import document_attributes_SPC_CLM_MASSA

urlpatterns = [

    path('Document_attributes_SPC_CLM_MASSA/Fetch/', document_attributes_SPC_CLM_MASSA.Document_attributes_SPC_CLM_MASSA_Fetch),
    path('Document_attributes_SPC_CLM_MASSA/Add', document_attributes_SPC_CLM_MASSA.Document_attributes_SPC_CLM_MASSA_Add),
    path('Document_attributes_SPC_CLM_MASSA/Update', document_attributes_SPC_CLM_MASSA.Document_attributes_SPC_CLM_MASSA_Update),
    path('Document_attributes_SPC_CLM_MASSA/Remove', document_attributes_SPC_CLM_MASSA.Document_attributes_SPC_CLM_MASSA_Remove),
    path('Document_attributes_SPC_CLM_MASSA/Lookup/', document_attributes_SPC_CLM_MASSA.Document_attributes_SPC_CLM_MASSA_Lookup),
    path('Document_attributes_SPC_CLM_MASSA/Info/', document_attributes_SPC_CLM_MASSA.Document_attributes_SPC_CLM_MASSA_Info),
    path('Document_attributes_SPC_CLM_MASSA/Copy', document_attributes_SPC_CLM_MASSA.Document_attributes_SPC_CLM_MASSA_Copy),

]



# Generated by Django 2.2.7 on 2019-11-17 11:27

import django.utils.timezone
from django.db import migrations, models

import isc_common.fields.description_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0158_delete_status'),
        ('production', '0089_auto_20191117_1110'),
    ]

    operations = [
        migrations.CreateModel(
            name='Launch_operations_item',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('qty', models.PositiveIntegerField(blank=True, default=None, null=True)),
                ('num', models.PositiveIntegerField(blank=True, default=None, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('ed_izm', isc_common.fields.related.ForeignKeyProtect(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm')),
                ('item', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item')),
                ('operation', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Operations')),
                ('operationitem', isc_common.fields.related.ForeignKeySetNull(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='production.Operations_item')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Launch_operations_item')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
                'unique_together': {('item', 'num')},
            },
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-07 18:10

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0159_auto_20191207_1801'),
        ('planing', '0039_auto_20191207_1802'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_launch_operation_material',
            name='edizm',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operation_material',
            name='material',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operation_material',
            name='material_askon',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operation_material',
            name='qty',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operation_resources',
            name='location',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operation_resources',
            name='resource',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operations_item',
            name='ed_izm',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operations_item',
            name='num',
        ),
        migrations.RemoveField(
            model_name='operation_launch_operations_item',
            name='qty',
        ),
        migrations.AddField(
            model_name='operation_launch_operation_material',
            name='launch_operation_material',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Launch_operations_material'),
        ),
        migrations.AddField(
            model_name='operation_launch_operation_resources',
            name='launch_operation_resources',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Launch_operation_resources'),
        ),
        migrations.AddField(
            model_name='operation_launch_operations_item',
            name='launch_operations_item',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Launch_operations_item'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-15 20:48

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0113_auto_20191215_1120'),
    ]

    operations = [
        migrations.CreateModel(
            name='Lotsman_documents_hierarcy_view',
            fields=[
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('parent_id', models.BigIntegerField(blank=True, null=True)),
                ('STMP_1', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('STMP_2', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('STMP_120', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('SPC_CLM_KOD', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('SPC_CLM_FORMAT', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('SPC_CLM_MASSA', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('SPC_CLM_POS', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('SPC_CLM_NAME', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('SPC_CLM_MARK', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('SPC_CLM_COUNT', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('Документ_на_материал', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('Наименование_материала', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('Документ_на_сортамент', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('Наименование_сортамента', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('section', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('subsection', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('isFolder', models.BooleanField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Иерархия документа из Лоцмана',
                'db_table': 'kd_lotsman_documents_hierarcy_mview',
                'managed': False,
            },
        ),
        migrations.AddField(
            model_name='documents_thumb',
            name='lotsman_document',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Lotsman_documents_hierarcy', verbose_name='Лоцман'),
        ),
        migrations.AddField(
            model_name='documents_thumb10',
            name='lotsman_document',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Lotsman_documents_hierarcy', verbose_name='Лоцман'),
        ),
        migrations.AlterField(
            model_name='documents_thumb',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Documents', verbose_name='КД'),
        ),
        migrations.AlterField(
            model_name='documents_thumb10',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Documents', verbose_name='КД'),
        ),
    ]
